@@ -1,8 +1,6 @@
-'''
-@anchor pydoc:grizzly.steps.scenario Scenario
-This package contains step implementations that only is allowed in the `Scenario` section in a feature file.
+"""Module contains step implementations that only is allowed in the `Scenario` section in a feature file.
 
-``` gherkin
+```gherkin
 Feature: Example
     Background:
         # Not here!
@@ -11,11 +9,10 @@ Feature: Example
 ```
 
 The steps in the `Scenario` section modifies the context only for the scenario that they are defined in.
-'''
+"""
 
-# flake8: noqa: F401,F403
-from .tasks import *
-from .setup import *
-from .user import *
-from .results import *
 from .response import *
+from .results import *
+from .setup import *
+from .tasks import *
+from .user import *
