@@ -1,6 +1,4 @@
-'''
-@anchor pydoc:grizzly.testdata.variables.date Date
-This variable is used to format and use dates.
+"""Format and create dates.
 
 ## Format
 
@@ -9,67 +7,78 @@ at the time of access.
 
 ## Arguments
 
-* `format` _str_ - a python [`strftime` format string](https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes), this argument is required
-* `timezone` _str_ (optional) - a valid [timezone name](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)
-* `offset` _str_ (optional) - a time span string describing the offset, Y = years, M = months, D = days, h = hours, m = minutes, s = seconds, e.g. `1Y-2M10D`
+| Name       | Type  | Description                                                                                                                               | Default        |
+| ---------- |------ | ----------------------------------------------------------------------------------------------------------------------------------------- | -------------- |
+| `format`   | `str` | python [`strftime` format string](https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes)                     | _required_     |
+| `timezone` | `str` | valid [timezone name](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)                                                       | `None` (local) |
+| `offset`   | `str` | time span string describing the offset, `Y` = years, `M` = months, `D` = days, `h` = hours, `m` = minutes, `s` = seconds, e.g. `1Y-2M10D` | `None`         |
 
 ## Example
 
-``` gherkin
+```gherkin
 And value for variable "AtomicDate.arrival" is "now | format='%Y-%m-%dT%H:%M:%S.000Z', timezone=UTC"
 ```
 
 This can then be used in a template:
 
-``` json
+```json
 {
     "arrival": "{{ AtomicDate.arrival }}",
     "location": "Port of Shanghai"
 }
 ```
-'''
-from typing import Union, Dict, Any, List, Type, Optional, cast
-from datetime import datetime
+"""
 
-from dateutil.parser import ParserError, parse as dateparse
+from __future__ import annotations
+
+from contextlib import suppress
+from datetime import datetime
+from typing import TYPE_CHECKING, ClassVar, cast
+
+from dateutil.parser import ParserError
+from dateutil.parser import parse as dateparse
 from dateutil.relativedelta import relativedelta
 
-try:
-    from zoneinfo import ZoneInfo, ZoneInfoNotFoundError  # pylint: disable=import-error
-except ImportError:
-    # pyright: reportMissingImports=false
-    from backports.zoneinfo import ZoneInfo, ZoneInfoNotFoundError  # type: ignore[no-redef]  # pylint: disable=import-error
-
-from grizzly_extras.arguments import split_value, parse_arguments
-
+from grizzly.types import StrDict, ZoneInfo, ZoneInfoNotFoundError
 from grizzly.utils import parse_timespan
+from grizzly_extras.arguments import parse_arguments, split_value
+from grizzly_extras.text import has_separator
+
 from . import AtomicVariable
 
+if TYPE_CHECKING:  # pragma: no cover
+    from grizzly.context import GrizzlyContextScenario
+
 
 def atomicdate__base_type__(value: str) -> str:
     if not isinstance(value, str):
-        raise ValueError(f'AtomicDate: {value} ({type(value)}) is not a string')
+        message = f'AtomicDate: {value} ({type(value)}) is not a string'  # type: ignore[unreachable]
+        raise TypeError(message)
 
-    if '|' in value:
+    if has_separator('|', value):
         date_value, date_arguments = split_value(value)
 
         try:
             arguments = parse_arguments(date_arguments)
         except ValueError as e:
-            raise ValueError(f'AtomicDate: {str(e)}') from e
+            message = f'AtomicDate: {e!s}'
+            raise ValueError(message) from e
 
-        for argument in arguments.keys():
+        for argument in arguments:
             if argument not in AtomicDate.arguments:
-                raise ValueError(f'AtomicDate: argument {argument} is not allowed')
+                message = f'AtomicDate: argument {argument} is not allowed'
+                raise ValueError(message)
 
         if 'format' not in arguments:
-            raise ValueError(f'AtomicDate: date format is not specified: "{value}"')
+            message = f'AtomicDate: date format is not specified: "{value}"'
+            raise ValueError(message)
 
         if 'timezone' in arguments:
             try:
                 ZoneInfo(arguments['timezone'])
-            except ZoneInfoNotFoundError:
-                raise ValueError(f'AtomicDate: unknown timezone "{arguments["timezone"]}"')
+            except ZoneInfoNotFoundError as e:
+                message = f'AtomicDate: unknown timezone "{arguments["timezone"]}"'
+                raise ValueError(message) from e
 
         if 'offset' in arguments:
             parse_timespan(arguments['offset'])
@@ -83,31 +92,34 @@ def atomicdate__base_type__(value: str) -> str:
 
     try:
         dateparse(date_value)
-
-        return value
     except (TypeError, ParserError) as e:
-        raise ValueError(f'AtomicDate: {str(e)}') from e
+        message = f'AtomicDate: {e!s}'
+        raise ValueError(message) from e
+    else:
+        return value
 
 
-class AtomicDate(AtomicVariable[Union[str, datetime]]):
+class AtomicDate(AtomicVariable[str | datetime]):
     __base_type__ = atomicdate__base_type__
     __initialized: bool = False
-    _settings: Dict[str, Dict[str, Any]]
-    arguments: Dict[str, Any] = {'format': str, 'timezone': str, 'offset': int}
+    _settings: dict[str, StrDict]
+    arguments: ClassVar[StrDict] = {'format': str, 'timezone': str, 'offset': int}
 
-    _special_variables: List[str] = ['now']
+    _special_variables: ClassVar[set[str]] = {'now'}
 
     def __init__(
         self,
+        *,
+        scenario: GrizzlyContextScenario,
         variable: str,
         value: str,
-        outer_lock: bool = False
+        outer_lock: bool = False,
     ) -> None:
-        with self.semaphore(outer_lock):
+        with self.semaphore(outer=outer_lock):
             initial_value: str
-            timezone: Optional[ZoneInfo] = None
+            timezone: ZoneInfo | None = None
             date_format = '%Y-%m-%d %H:%M:%S'
-            offset: Optional[Dict[str, int]] = None
+            offset: dict[str, int] | None = None
 
             safe_value = self.__class__.__base_type__(value)
 
@@ -132,14 +144,9 @@ class AtomicDate(AtomicVariable[Union[str, datetime]]):
                 'offset': offset,
             }
 
-            date_value: Union[str, datetime]
+            date_value: str | datetime = 'now' if initial_value is None or len(initial_value) < 1 or initial_value == 'now' else dateparse(initial_value)
 
-            if initial_value is None or len(initial_value) < 1 or initial_value == 'now':
-                date_value = 'now'
-            else:
-                date_value = dateparse(initial_value)
-
-            super().__init__(variable, date_value, outer_lock=True)
+            super().__init__(scenario=scenario, variable=variable, value=date_value, outer_lock=True)
 
             if self.__initialized:
                 self._settings[variable] = settings
@@ -150,15 +157,17 @@ class AtomicDate(AtomicVariable[Union[str, datetime]]):
             self.__initialized = True
 
     @classmethod
-    def clear(cls: Type['AtomicDate']) -> None:
+    def clear(cls: type[AtomicDate]) -> None:
         super().clear()
 
-        instance = cast(AtomicDate, cls.get())
-        variables = list(instance._settings.keys())
-        for variable in variables:
-            del instance._settings[variable]
+        instances = cls._instances.get(cls, {})
+        for scenario in instances:
+            instance = cast('AtomicDate', cls.get(scenario))
+            variables = list(instance._settings.keys())
+            for variable in variables:
+                del instance._settings[variable]
 
-    def __getitem__(self, variable: str) -> Optional[str]:
+    def __getitem__(self, variable: str) -> str | None:
         with self.semaphore():
             value = self._get_value(variable)
 
@@ -169,7 +178,8 @@ class AtomicDate(AtomicVariable[Union[str, datetime]]):
             elif isinstance(value, datetime):
                 date_value = value
             else:
-                raise ValueError(f'{self.__class__.__name__}.{variable} was incorrectly initialized with "{value}"')
+                message = f'{self.__class__.__name__}.{variable} was incorrectly initialized with "{value}"'
+                raise ValueError(message)
 
             offset = self._settings[variable]['offset']
 
@@ -182,9 +192,7 @@ class AtomicDate(AtomicVariable[Union[str, datetime]]):
 
     def __delitem__(self, variable: str) -> None:
         with self.semaphore():
-            try:
+            with suppress(KeyError):
                 del self._settings[variable]
-            except KeyError:
-                pass
 
             super().__delitem__(variable)
