@@ -1,6 +1,4 @@
-'''
-@anchor pydoc:grizzly.testdata.variables.directory_contents Directory Contents
-This variable provides a list of files in the specified directory.
+"""Provide a list of files in the specified directory.
 
 ## Format
 
@@ -8,14 +6,16 @@ Relative path of a directory under `requests/`.
 
 ## Arguments
 
-* `repeat` _bool_ (optional) - wether values should be reused, e.g. when reaching the end it should start from the beginning again (default: `False`)
-* `random` _bool_ (optional) - if files should be selected by random, instead of sequential from first to last (default: `False`)
+| Name     | Type   | Description                                                                                          | Default |
+| -------- | ------ | ---------------------------------------------------------------------------------------------------- | ------- |
+| `repeat` | `bool` | whether values should be reused, e.g. when reaching the end it should start from the beginning again | `False` |
+| `random` | `bool` | if files should be selected by random, instead of sequential from first to last                      | `False` |
 
 ## Example
 
 With the following directory structure:
 
-``` plain
+```plain
 .
 └── requests
     └── files
@@ -26,50 +26,59 @@ With the following directory structure:
         └── file5.bin
 ```
 
-``` gherkin
+```gherkin
 And value for variable "AtomicDirectoryContents.files" is "files/ | repeat=True, random=False"
 And put request "{{ AtomicDirectoryContents.files }}" with name "put-file" to endpoint "/tmp"
 ```
 
 First request will provide `file1.bin`, second `file2.bin` etc.
-'''
-import os
+"""
 
-from typing import Dict, List, Any, Type, Optional, cast
+from __future__ import annotations
+
+from contextlib import suppress
+from os import environ
 from pathlib import Path
-from random import randint
+from secrets import randbelow
+from typing import TYPE_CHECKING, ClassVar, cast
 
-from grizzly_extras.arguments import split_value, parse_arguments
-
-from grizzly.types import bool_type
+from grizzly.types import StrDict, bool_type
+from grizzly_extras.arguments import parse_arguments, split_value
+from grizzly_extras.text import has_separator
 
 from . import AtomicVariable
 
+if TYPE_CHECKING:  # pragma: no cover
+    from grizzly.context import GrizzlyContextScenario
+
 
 def atomicdirectorycontents__base_type__(value: str) -> str:
-    grizzly_context_requests = os.path.join(os.environ.get('GRIZZLY_CONTEXT_ROOT', ''), 'requests')
-    if '|' in value:
+    grizzly_context_requests = Path(environ.get('GRIZZLY_CONTEXT_ROOT', '')) / 'requests'
+    if has_separator('|', value):
         [directory_value, directory_arguments] = split_value(value)
 
         try:
             arguments = parse_arguments(directory_arguments)
         except ValueError as e:
-            raise ValueError(f'AtomicDirectoryContents: {str(e)}') from e
+            message = f'AtomicDirectoryContents: {e!s}'
+            raise ValueError(message) from e
 
         for argument_name, argument_value in arguments.items():
             if argument_name not in AtomicDirectoryContents.arguments:
-                raise ValueError(f'AtomicDirectoryContents: argument {argument_name} is not allowed')
-            else:
-                AtomicDirectoryContents.arguments[argument_name](argument_value)
+                message = f'AtomicDirectoryContents: argument {argument_name} is not allowed'
+                raise ValueError(message)
+
+            AtomicDirectoryContents.arguments[argument_name](argument_value)
 
         value = f'{directory_value} | {directory_arguments}'
     else:
         directory_value = value
 
-    path = os.path.join(grizzly_context_requests, directory_value)
+    path = grizzly_context_requests / directory_value
 
-    if not os.path.isdir(path):
-        raise ValueError(f'AtomicDirectoryContents: {directory_value} is not a directory in {grizzly_context_requests}')
+    if not path.is_dir():
+        message = f'AtomicDirectoryContents: {directory_value} is not a directory in {grizzly_context_requests!s}'
+        raise ValueError(message)
 
     return value
 
@@ -78,23 +87,25 @@ class AtomicDirectoryContents(AtomicVariable[str]):
     __base_type__ = atomicdirectorycontents__base_type__
     __initialized: bool = False
 
-    _files: Dict[str, List[str]]
-    _settings: Dict[str, Dict[str, Any]]
-    _requests_context_root: str
-    arguments: Dict[str, Any] = {'repeat': bool_type, 'random': bool_type}
+    _files: dict[str, list[str]]
+    _settings: dict[str, StrDict]
+    _requests_context_root: Path
+    arguments: ClassVar[StrDict] = {'repeat': bool_type, 'random': bool_type}
 
     def __init__(
         self,
+        *,
+        scenario: GrizzlyContextScenario,
         variable: str,
         value: str,
-        outer_lock: bool = False
-    ):
-        with self.semaphore(outer_lock):
+        outer_lock: bool = False,
+    ) -> None:
+        with self.semaphore(outer=outer_lock):
             safe_value = self.__class__.__base_type__(value)
 
             settings = {'repeat': False, 'random': False}
 
-            if '|' in safe_value:
+            if has_separator('|', safe_value):
                 directory, directory_arguments = split_value(safe_value)
 
                 arguments = parse_arguments(directory_arguments)
@@ -105,7 +116,7 @@ class AtomicDirectoryContents(AtomicVariable[str]):
             else:
                 directory = safe_value
 
-            super().__init__(variable, directory, outer_lock=True)
+            super().__init__(scenario=scenario, variable=variable, value=directory, outer_lock=True)
 
             if self.__initialized:
                 if variable not in self._files:
@@ -116,34 +127,32 @@ class AtomicDirectoryContents(AtomicVariable[str]):
 
                 return
 
-            self._requests_context_root = os.path.join(os.environ.get('GRIZZLY_CONTEXT_ROOT', '.'), 'requests')
+            self._requests_context_root = Path(environ.get('GRIZZLY_CONTEXT_ROOT', '.')) / 'requests'
             self._files = {variable: self._create_file_queue(directory)}
             self._settings = {variable: settings}
             self.__initialized = True
 
     @classmethod
-    def clear(cls: Type['AtomicDirectoryContents']) -> None:
+    def clear(cls: type[AtomicDirectoryContents]) -> None:
         super().clear()
 
-        instance = cast(AtomicDirectoryContents, cls.get())
-        variables = list(instance._files.keys())
+        instances = cls._instances.get(cls, {})
+        for scenario in instances:
+            instance = cast('AtomicDirectoryContents', cls.get(scenario))
+            variables = list(instance._files.keys())
 
-        for variable in variables:
-            del instance._files[variable]
-            del instance._settings[variable]
+            for variable in variables:
+                del instance._files[variable]
+                del instance._settings[variable]
 
-    def _create_file_queue(self, directory: str) -> List[str]:
-        parent_part = len(self._requests_context_root) + 1
-        queue = [
-            str(path)[parent_part:]
-            for path in Path(os.path.join(self._requests_context_root, directory)).rglob('*')
-            if path.is_file()
-        ]
+    def _create_file_queue(self, directory: str) -> list[str]:
+        parent_part = len(str(self._requests_context_root)) + 1
+        queue = [str(path)[parent_part:] for path in (self._requests_context_root / directory).rglob('*') if path.is_file()]
         queue.sort()
 
         return queue
 
-    def __getitem__(self, variable: str) -> Optional[str]:
+    def __getitem__(self, variable: str) -> str | None:
         with self.semaphore():
             self._get_value(variable)
 
@@ -151,8 +160,8 @@ class AtomicDirectoryContents(AtomicVariable[str]):
                 settings = self._settings[variable]
 
                 if settings['random'] is True:
-                    roof = len(self._files[variable]) - 1
-                    index = randint(0, roof)
+                    roof = len(self._files[variable])
+                    index = randbelow(roof)
                 else:
                     index = 0
 
@@ -160,21 +169,17 @@ class AtomicDirectoryContents(AtomicVariable[str]):
 
                 if settings['repeat'] is True:
                     self._files[variable].append(value)
-
-                return value
             except (IndexError, ValueError):
                 return None
+            else:
+                return value
 
     def __delitem__(self, variable: str) -> None:
         with self.semaphore():
-            try:
+            with suppress(KeyError):
                 del self._files[variable]
-            except KeyError:
-                pass
 
-            try:
+            with suppress(KeyError):
                 del self._settings[variable]
-            except KeyError:
-                pass
 
             super().__delitem__(variable)
