@@ -1,6 +1,4 @@
-'''
-@anchor pydoc:grizzly.testdata.variables.integer_incrementer Integer Incrementer
-This variable provides an unique integer each time it is accessed.
+"""Unique incremented integer each time it is accessed.
 
 Useful to generate unique ID for each request.
 
@@ -10,20 +8,21 @@ The first value of an integer that is going to be used.
 
 ## Arguments
 
-* `step` _int_, (optional) - how much the value should increment each time (default `1`)
-
-* `persist` _bool_, (optional) - if the initial value should be persist and loaded from file (default `False`)
+| Name      | Type   | Description                                                 | Default |
+| --------- | ------ | ----------------------------------------------------------- | ------- |
+| `step`    | `int`  | how much the value should increment each time               | `1`     |
+| `persist` | `bool` | if the initial value should be persist and loaded from file | `False` |
 
 ## Example
 
-``` gherkin title="example.feature"
+```gherkin title="example.feature"
 And value for variable "AtomicIntegerIncrementer.unique_id" is "100 | step=10"
 And value for variable "AtomicIntegerIncrementer.persistent" is "10 | step=5, persist=True"
 ```
 
 This can then be used in a template:
 
-``` json
+```json
 {
     "id": {{ AtomicIntegerIncrementer.unique_id }}
 }
@@ -51,7 +50,7 @@ Values of `AtomicIntegerIncrementer.unique_id`, per run and iteration:
 
     4. ...
 
-Values of `AtomicIntegerIncrementer.persistent`, per run and iteration:
+Values of `AtomicIntegerIncrementer.persistent`, per run and iteration, for 4 iterations:
 
 1. Run (`features/persistent/example.json` missing)
 
@@ -61,50 +60,61 @@ Values of `AtomicIntegerIncrementer.persistent`, per run and iteration:
 
     3. `20`
 
-    4. ...
+    4. `25`
 
 2. Run (`features/persistent/example.json` created by Run 1, due to `persistent=True`), initial
-value `35 | step=5, persist=True` will be read from the file and override what is written in `example.feature`
+value `25 | step=5, persist=True` will be read from the file and override what is written in `example.feature`
 
-    1. `25`
+    1. `30`
 
-    2. `30`
+    2. `35`
 
-    3. `35`
+    3. `40`
 
-    4. ...
-'''
-from typing import Union, Dict, Any, Type, Optional, cast
+    4. `45`
+"""
 
-from grizzly_extras.arguments import split_value, parse_arguments
+from __future__ import annotations
 
-from grizzly.types import bool_type
+from contextlib import suppress
+from typing import TYPE_CHECKING, ClassVar, cast
+
+from grizzly.types import StrDict, bool_type
+from grizzly_extras.arguments import parse_arguments, split_value
+from grizzly_extras.text import has_separator
 
 from . import AtomicVariable, AtomicVariablePersist
 
+if TYPE_CHECKING:  # pragma: no cover
+    from grizzly.context import GrizzlyContextScenario
 
-def atomicintegerincrementer__base_type__(value: Union[str, int]) -> str:
+
+def atomicintegerincrementer__base_type__(value: str | int) -> str:
     if isinstance(value, int):
         return str(value)
 
-    if '|' in value:
+    if has_separator('|', value):
         try:
             initial_value, incrementer_arguments = split_value(value)
             initial_value = str(int(float(initial_value)))
         except ValueError as e:
-            raise ValueError(f'AtomicIntegerIncrementer: "{value}" is not a valid initial value') from e
+            message = f'AtomicIntegerIncrementer: "{value}" is not a valid initial value'
+            raise ValueError(message) from e
 
         try:
             arguments = parse_arguments(incrementer_arguments)
         except ValueError as e:
-            raise ValueError(f'AtomicIntegerIncrementer: {str(e)}') from e
+            message = f'AtomicIntegerIncrementer: {e!s}'
+            raise ValueError(message) from e
 
         if 'step' not in arguments:
-            raise ValueError(f'AtomicIntegerIncrementer: step is not specified: "{value}"')
+            message = f'AtomicIntegerIncrementer: step is not specified: "{value}"'
+            raise ValueError(message)
 
         for argument in arguments:
             if argument not in AtomicIntegerIncrementer.arguments:
-                raise ValueError(f'AtomicIntegerIncrementer: argument {argument} is not allowed')
+                message = f'AtomicIntegerIncrementer: argument {argument} is not allowed'
+                raise ValueError(message)
 
             AtomicIntegerIncrementer.arguments[argument](arguments[argument])
 
@@ -113,7 +123,8 @@ def atomicintegerincrementer__base_type__(value: Union[str, int]) -> str:
         try:
             value = str(int(float(value.strip())))
         except ValueError as e:
-            raise ValueError(f'AtomicIntegerIncrementer: "{value}" is not a valid initial value') from e
+            message = f'AtomicIntegerIncrementer: "{value}" is not a valid initial value'
+            raise ValueError(message) from e
 
     return value
 
@@ -122,14 +133,14 @@ class AtomicIntegerIncrementer(AtomicVariable[int], AtomicVariablePersist):
     __base_type__ = atomicintegerincrementer__base_type__
 
     __initialized: bool = False
-    _steps: Dict[str, Any]
-    arguments: Dict[str, Any] = {'step': int, 'persist': bool_type}
+    _steps: dict
+    arguments: ClassVar[StrDict] = {'step': int, 'persist': bool_type}
 
-    def __init__(self, variable: str, value: Union[str, int], outer_lock: bool = False) -> None:
-        with self.semaphore(outer_lock):
+    def __init__(self, *, scenario: GrizzlyContextScenario, variable: str, value: str | int, outer_lock: bool = False) -> None:
+        with self.semaphore(outer=outer_lock):
             safe_value = self.__class__.__base_type__(value)
 
-            if '|' in safe_value:
+            if has_separator('|', safe_value):
                 incrementer_value, incrementer_arguments = split_value(safe_value)
                 initial_value = incrementer_value
                 arguments = parse_arguments(incrementer_arguments)
@@ -140,7 +151,7 @@ class AtomicIntegerIncrementer(AtomicVariable[int], AtomicVariablePersist):
                 step = 1
                 state = False
 
-            super().__init__(variable, int(initial_value), outer_lock=True)
+            super().__init__(scenario=scenario, variable=variable, value=int(initial_value), outer_lock=True)
 
             if self.__initialized:
                 if variable not in self._steps:
@@ -152,10 +163,12 @@ class AtomicIntegerIncrementer(AtomicVariable[int], AtomicVariablePersist):
             self.__initialized = True
 
     def generate_initial_value(self, variable: str) -> str:
+        """Generate next, persistent, initialization value."""
         persist = self._steps.get(variable, {}).get('persist', False)
 
         if not persist:
-            raise ValueError(f'{self.__class__.__name__}.{variable} should not be persisted')
+            message = f'{self.__class__.__name__}.{variable} should not be persisted'
+            raise ValueError(message)
 
         value = self.__getitem__(variable)
         arguments = ', '.join([f'{key}={value}' for key, value in self._steps[variable].items()])
@@ -163,15 +176,17 @@ class AtomicIntegerIncrementer(AtomicVariable[int], AtomicVariablePersist):
         return f'{value} | {arguments}'
 
     @classmethod
-    def clear(cls: Type['AtomicIntegerIncrementer']) -> None:
+    def clear(cls: type[AtomicIntegerIncrementer]) -> None:
         super().clear()
 
-        instance = cast(AtomicIntegerIncrementer, cls.get())
-        variables = list(instance._steps.keys())
-        for variable in variables:
-            del instance._steps[variable]
+        instances = cls._instances.get(cls, {})
+        for scenario in instances:
+            instance = cast('AtomicIntegerIncrementer', cls.get(scenario))
+            variables = list(instance._steps.keys())
+            for variable in variables:
+                del instance._steps[variable]
 
-    def __getitem__(self, variable: str) -> Optional[int]:
+    def __getitem__(self, variable: str) -> int | None:
         with self.semaphore():
             value = self._get_value(variable)
 
@@ -182,9 +197,7 @@ class AtomicIntegerIncrementer(AtomicVariable[int], AtomicVariablePersist):
 
     def __delitem__(self, variable: str) -> None:
         with self.semaphore():
-            try:
+            with suppress(KeyError):
                 del self._steps[variable]
-            except KeyError:
-                pass
 
             super().__delitem__(variable)
