@@ -1,6 +1,4 @@
-'''
-@anchor pydoc:grizzly.testdata.variables.random_string Random String
-This variable generates a specified number of unique strings, based on a string format pattern.
+"""Generate a specified number of unique strings, based on a string format pattern.
 
 The list is pre-populated to ensure that each string is unique.
 
@@ -10,60 +8,74 @@ Initial value is a string pattern specified with `%s` and `%d`, or `%g`.
 
 * `%s` represents **one** ASCII letter
 * `%d` represents **one** digit between `0` and `9`
-* `%g` represents one complete UUID, cannot be combined with other string patterns
+* `%g` represents one complete UUID(4), cannot be combined with other string patterns
 
 Parts of the string can be static, e.g. not random.
 
 ## Arguments
 
-* `count` _int_ (optional) - number of unique strings to generate (default: `1`)
-* `upper` _bool_ (optional) - if the strings should be in upper case (default: `False`)
+| Name    | Type   | Description                            | Default |
+| ------- | ------ | -------------------------------------- | ------- |
+| `count` | `int`  | number of unique strings to generate   | `1`     |
+| `upper` | `bool` | if the strings should be in upper case | `False` |
 
 ## Example
 
-``` gherkin
+```gherkin
 And value for variable "AtomicRandomString.registration_plate_number" is "%s%sZ%d%d0 | upper=True, count=100"
 And value for variable "AtomicRandomString.uuid" is "%g | count=100"
 ```
 
 This can then be used in a template:
-``` json
+```json
 {
     "registration_plate_number": "{{ AtomicRandomString.registration_plate_number }}"
 }
 ```
 
 `AtomicRandomString.registration_plate_number` will then be a string in the format `[A-Z][A-Z]Z[0-9][0-9]0` and there will be `100` unique values for disposal.
-'''
-from typing import Dict, List, Any, Callable, Optional, Set, Type, cast
-from random import randint, choice
+"""
+
+from __future__ import annotations
+
+from contextlib import suppress
+from secrets import choice, randbelow
 from string import ascii_letters
+from typing import TYPE_CHECKING, ClassVar, cast
 from uuid import uuid4
 
-from grizzly_extras.arguments import split_value, parse_arguments
-
-from grizzly.types import bool_type, int_rounded_float_type
+from grizzly.types import StrDict, bool_type, int_rounded_float_type
+from grizzly_extras.arguments import parse_arguments, split_value
+from grizzly_extras.text import has_separator
 
 from . import AtomicVariable
 
+if TYPE_CHECKING:  # pragma: no cover
+    from collections.abc import Callable
+
+    from grizzly.context import GrizzlyContextScenario
+
 
 def atomicrandomstring__base_type__(value: str) -> str:
     if len(value) < 1:
-        raise ValueError('AtomicRandomString: no string pattern specified')
+        message = 'AtomicRandomString: no string pattern specified'
+        raise ValueError(message)
 
-    if '|' in value:
+    if has_separator('|', value):
         string_pattern, string_arguments = split_value(value)
 
         try:
             arguments = parse_arguments(string_arguments)
         except ValueError as e:
-            raise ValueError(f'AtomicRandomString: {str(e)}') from e
+            message = f'AtomicRandomString: {e!s}'
+            raise ValueError(message) from e
 
         for argument, v in arguments.items():
             if argument not in AtomicRandomString.arguments:
-                raise ValueError(f'AtomicRandomString: argument {argument} is not allowed')
-            else:
-                AtomicRandomString.arguments[argument](v)
+                message = f'AtomicRandomString: argument {argument} is not allowed'
+                raise ValueError(message)
+
+            AtomicRandomString.arguments[argument](v)
 
         value = f'{string_pattern} | {string_arguments}'
     else:
@@ -72,10 +84,12 @@ def atomicrandomstring__base_type__(value: str) -> str:
     generators = AtomicRandomString.get_generators(string_pattern)
 
     if len(generators) < 1:
-        raise ValueError('AtomicRandomString: specified string pattern does not contain any generators')
+        message = 'AtomicRandomString: specified string pattern does not contain any generators'
+        raise ValueError(message)
 
     if '%g' in string_pattern and string_pattern.count('%') != 1:
-        raise ValueError('AtomicRandomString: %g cannot be combined with other formats')
+        message = 'AtomicRandomString: %g cannot be combined with other formats'
+        raise ValueError(message)
 
     return value
 
@@ -84,30 +98,32 @@ class AtomicRandomString(AtomicVariable[str]):
     __base_type__ = atomicrandomstring__base_type__
     __initialized: bool = False
 
-    _strings: Dict[str, List[str]]
-    arguments: Dict[str, Any] = {'upper': bool_type, 'count': int_rounded_float_type}
+    _strings: dict[str, list[str]]
+    arguments: ClassVar[StrDict] = {'upper': bool_type, 'count': int_rounded_float_type}
 
     @staticmethod
-    def get_generators(format: str) -> List[Callable[['AtomicRandomString'], str]]:
-        formats: List[Callable[[AtomicRandomString], str]] = []
+    def get_generators(format_string: str) -> list[Callable[[AtomicRandomString], str]]:
+        """Map format modifiers to generator functions."""
+        formats: list[Callable[[AtomicRandomString], str]] = []
         # first item is either empty, or it's a static character
-        for f in format.split('%')[1:]:
-            f = f[0]  # could be static characters in the pattern, only supports one character formatters
-            generator = getattr(AtomicRandomString, f'_generate_{f}', None)
+        for format_modifier in format_string.split('%')[1:]:
+            generator_name = format_modifier[0]  # could be static characters in the pattern, only supports one character formatters
+            generator = getattr(AtomicRandomString, f'_generate_{generator_name}', None)
             if not callable(generator):
-                raise NotImplementedError(f'AtomicRandomString: format "{f}" is not implemented')
+                message = f'AtomicRandomString: format "{generator_name}" is not implemented'
+                raise NotImplementedError(message)
 
             formats.append(generator)
 
         return formats
 
-    def __init__(self, variable: str, value: str, outer_lock: bool = False) -> None:
-        with self.semaphore(outer_lock):
+    def __init__(self, *, scenario: GrizzlyContextScenario, variable: str, value: str, outer_lock: bool = False) -> None:
+        with self.semaphore(outer=outer_lock):
             safe_value = self.__class__.__base_type__(value)
 
             settings = {'upper': False, 'count': 1}
 
-            if '|' in safe_value:
+            if has_separator('|', safe_value):
                 string_pattern, string_arguments = split_value(safe_value)
 
                 arguments = parse_arguments(string_arguments)
@@ -116,12 +132,11 @@ class AtomicRandomString(AtomicVariable[str]):
                     if argument in arguments:
                         settings[argument] = caster(arguments[argument])
 
-                if settings['count'] < 1:
-                    settings['count'] = 1
+                settings['count'] = max(settings['count'], 1)
             else:
                 string_pattern = value
 
-            super().__init__(variable, string_pattern, outer_lock=True)
+            super().__init__(scenario=scenario, variable=variable, value=string_pattern, outer_lock=True)
 
             if self.__initialized:
                 if variable not in self._strings:
@@ -136,27 +151,23 @@ class AtomicRandomString(AtomicVariable[str]):
         return choice(ascii_letters)
 
     def _generate_d(self) -> int:
-        return randint(0, 9)
+        return randbelow(10)
 
     def _generate_g(self) -> str:
         return str(uuid4())
 
-    def _generate_strings(self, string_pattern: str, settings: Dict[str, Any]) -> List[str]:
-        generated_strings: Set[str] = set()
+    def _generate_strings(self, string_pattern: str, settings: StrDict) -> list[str]:
+        generated_strings: set[str] = set()
         generators = self.__class__.get_generators(string_pattern)
 
         string_pattern = string_pattern.replace('%g', '%s')
 
-        for _ in range(0, settings['count']):
-            generated_string: Optional[str] = None
+        for _ in range(settings['count']):
+            generated_string: str | None = None
 
             while generated_string is None or generated_string in generated_strings:
-                generated_part: List[str] = []
-
-                for generator in generators:
-                    generated_part.append(generator(self))
-
-                generated_string = string_pattern % tuple(generated_part)
+                generated_part = tuple([generator(self) for generator in generators])
+                generated_string = string_pattern % generated_part
 
                 if settings['upper']:
                     generated_string = generated_string.upper()
@@ -166,16 +177,18 @@ class AtomicRandomString(AtomicVariable[str]):
         return list(generated_strings)
 
     @classmethod
-    def clear(cls: Type['AtomicRandomString']) -> None:
+    def clear(cls: type[AtomicRandomString]) -> None:
         super().clear()
 
-        instance = cast(AtomicRandomString, cls.get())
-        variables = list(instance._strings.keys())
+        instances = cls._instances.get(cls, {})
+        for scenario in instances:
+            instance = cast('AtomicRandomString', cls.get(scenario))
+            variables = list(instance._strings.keys())
 
-        for variable in variables:
-            del instance._strings[variable]
+            for variable in variables:
+                del instance._strings[variable]
 
-    def __getitem__(self, variable: str) -> Optional[str]:
+    def __getitem__(self, variable: str) -> str | None:
         with self.semaphore():
             self._get_value(variable)
 
@@ -186,9 +199,7 @@ class AtomicRandomString(AtomicVariable[str]):
 
     def __delitem__(self, variable: str) -> None:
         with self.semaphore():
-            try:
+            with suppress(KeyError):
                 del self._strings[variable]
-            except KeyError:
-                pass
 
             super().__delitem__(variable)
