@@ -1,24 +1,25 @@
-from typing import Callable, Union
+"""Locust types frequently used in grizzly."""
+
+from collections.abc import Callable
 
-from locust.runners import MasterRunner, WorkerRunner, LocalRunner
 from locust.env import Environment
+from locust.exception import CatchResponseError, LocustError, StopUser
 from locust.rpc.protocol import Message
-from locust.exception import StopUser, LocustError, CatchResponseError
-
+from locust.runners import LocalRunner, MasterRunner, WorkerRunner
 
 MessageHandler = Callable[[Environment, Message], None]
 
-LocustRunner = Union[MasterRunner, WorkerRunner, LocalRunner]
+LocustRunner = MasterRunner | WorkerRunner | LocalRunner
 
 __all__ = [
-    'MasterRunner',
-    'WorkerRunner',
-    'LocalRunner',
-    'LocustRunner',
-    'Environment',
-    'Message',
-    'StopUser',
-    'LocustError',
     'CatchResponseError',
+    'Environment',
+    'LocalRunner',
+    'LocustError',
+    'LocustRunner',
+    'MasterRunner',
+    'Message',
     'MessageHandler',
+    'StopUser',
+    'WorkerRunner',
 ]
