@@ -1,14 +1,20 @@
-import logging
+"""Unit tests of grizzly.tasks.log_message."""
 
-from _pytest.logging import LogCaptureFixture
+from __future__ import annotations
+
+import logging
+from typing import TYPE_CHECKING
 
 from grizzly.tasks import LogMessageTask
 
-from tests.fixtures import GrizzlyFixture
+if TYPE_CHECKING:  # pragma: no cover
+    from _pytest.logging import LogCaptureFixture
+
+    from tests.fixtures import GrizzlyFixture
 
 
 class TestLogMessageTask:
-    def test(self, grizzly_fixture: GrizzlyFixture, caplog: LogCaptureFixture) -> None:
+    def test_task(self, grizzly_fixture: GrizzlyFixture, caplog: LogCaptureFixture) -> None:
         task_factory = LogMessageTask(message='hello world!')
         assert task_factory.message == 'hello world!'
         assert task_factory.__template_attributes__ == {'message'}
@@ -17,12 +23,10 @@ class TestLogMessageTask:
 
         assert callable(task)
 
-        _, _, scenario = grizzly_fixture()
-
-        assert scenario is not None
+        parent = grizzly_fixture()
 
         with caplog.at_level(logging.INFO):
-            task(scenario)
+            task(parent)
         assert 'hello world!' in caplog.text
         caplog.clear()
 
@@ -33,9 +37,9 @@ class TestLogMessageTask:
 
         assert callable(task)
 
-        scenario.user._context['variables']['variable'] = 'hello world!'
+        parent.user.set_variable('variable', 'hello world!')
 
         with caplog.at_level(logging.INFO):
-            task(scenario)
+            task(parent)
         assert 'variable=hello world!' in caplog.text
         caplog.clear()
