@@ -1,23 +1,26 @@
+"""Unit tests of grizzly.tasks.request."""
+
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 import pytest
 
-from typing import Any, Tuple, Optional
-
-from pytest_mock import MockerFixture
-from locust.clients import ResponseContextManager
-from jinja2 import Template
-
-from grizzly.users.base.grizzly_user import GrizzlyUser
-from grizzly.users.base.response_handler import ResponseHandlerAction
-
-from grizzly_extras.transformer import TransformerContentType
+from grizzly.events.response_handler import ResponseHandlerAction
 from grizzly.tasks import (
     RequestTask,
     RequestTaskHandlers,
     RequestTaskResponse,
 )
 from grizzly.types import RequestMethod
+from grizzly_extras.transformer import TransformerContentType
 
-from tests.fixtures import GrizzlyFixture
+if TYPE_CHECKING:  # pragma: no cover
+    from pytest_mock import MockerFixture
+
+    from grizzly.types import HandlerContextType
+    from grizzly.users import GrizzlyUser
+    from tests.fixtures import GrizzlyFixture
 
 
 class TestRequestTaskHandlers:
@@ -31,8 +34,8 @@ class TestRequestTaskHandlers:
         assert len(handlers.payload) == 0
 
         class TestResponseHandlerAction(ResponseHandlerAction):
-            def __call__(self, input_context: Tuple[TransformerContentType, Any], user: GrizzlyUser, response: Optional[ResponseContextManager] = None) -> None:
-                super().__call__(input_context, user, response)
+            def __call__(self, input_context: tuple[TransformerContentType, HandlerContextType], user: GrizzlyUser) -> None:
+                super().__call__(input_context, user)
 
         handler = TestResponseHandlerAction(expression='', match_with='')
 
@@ -57,14 +60,14 @@ class TestRequestTaskResponse:
 
         response_task.add_status_code(200)
         response_task.add_status_code(302)
-        assert [200, 302] == response_task.status_codes
+        assert response_task.status_codes == [200, 302]
 
         response_task.add_status_code(200)
-        assert [200, 302] == response_task.status_codes
+        assert response_task.status_codes == [200, 302]
 
         response_task.add_status_code(-302)
         response_task.add_status_code(400)
-        assert [200, 400] == response_task.status_codes
+        assert response_task.status_codes == [200, 400]
 
 
 class TestRequestTask:
@@ -80,27 +83,21 @@ class TestRequestTask:
         assert not hasattr(task_factory, 'scenario')
 
         assert task_factory.source is None
-        assert task_factory.template is None
 
         task = task_factory()
         assert callable(task)
 
-        _, _, scenario = grizzly_fixture()
+        parent = grizzly_fixture()
 
-        assert scenario is not None
+        mocker.patch.object(parent.user, 'request')
+        request_spy = mocker.spy(parent.user, 'request')
 
-        mocker.patch.object(scenario.user, 'request', autospec=True)
-        request_spy = mocker.spy(scenario.user, 'request')
+        task(parent)
 
-        task(scenario)
-
-        assert request_spy.call_count == 1
-        args, _ = request_spy.call_args_list[0]
-        assert args[0] is task_factory
+        request_spy.assert_called_once_with(task_factory)
 
         # automagically create template if not set
         task_factory.source = 'hello {{ world }}'
-        assert isinstance(task_factory.template, Template)
 
     def test_arguments(self) -> None:
         task_factory = RequestTask(RequestMethod.GET, 'test-name', endpoint='/api/test | content_type="application/json", foo=bar')
@@ -116,19 +113,22 @@ class TestRequestTask:
         assert task_factory.response.content_type == TransformerContentType.XML
 
         # test missing required arguments for multipart/form-data
-        with pytest.raises(ValueError):
+        with pytest.raises(AssertionError, match=r'Content type multipart\/form-data requires endpoint arguments multipart_form_data_name and multipart_form_data_filename'):
             RequestTask(RequestMethod.POST, 'test-name', endpoint='/api/test | content_type="multipart/form-data"')
 
         # test required arguments for multipart/form-data
-        task_factory = RequestTask(RequestMethod.POST, 'test-name',
-                                   endpoint='/api/test | content_type="multipart/form-data", multipart_form_data_filename="foo", multipart_form_data_name="bar"')
+        task_factory = RequestTask(
+            RequestMethod.POST,
+            'test-name',
+            endpoint='/api/test | content_type="multipart/form-data", multipart_form_data_filename="foo", multipart_form_data_name="bar"',
+        )
         assert task_factory.endpoint == '/api/test'
         assert task_factory.arguments == {'multipart_form_data_filename': 'foo', 'multipart_form_data_name': 'bar'}
         assert task_factory.response.content_type == TransformerContentType.MULTIPART_FORM_DATA
 
     def test_add_metadata(self) -> None:
         task_factory = RequestTask(RequestMethod.GET, 'test-name', endpoint='/api/test | content_type="application/json", foo=bar')
-        assert getattr(task_factory, 'metadata', '') is None
+        assert getattr(task_factory, 'metadata', None) == {}
 
         task_factory.add_metadata('foo', 'bar')
         task_factory.add_metadata('alice', 'bob')
