@@ -1,21 +1,20 @@
-import shutil
+"""Unit tests for grizzly.context."""
 
-from typing import Tuple, Dict, Any, Optional, cast
-from os import path, environ
+from __future__ import annotations
+
+from contextlib import suppress
+from os import environ
+from typing import TYPE_CHECKING, Any, cast
 
 import pytest
+from gevent.lock import Semaphore
 
-from _pytest.tmpdir import TempPathFactory
-from jinja2.environment import Template
-
-from grizzly.types.behave import Scenario
-from grizzly.types.locust import Environment, Message
 from grizzly.context import (
     GrizzlyContext,
-    GrizzlyContextScenarios,
-    GrizzlyContextSetup,
     GrizzlyContextScenario,
+    GrizzlyContextScenarios,
     GrizzlyContextScenarioValidation,
+    GrizzlyContextSetup,
     GrizzlyContextSetupLocust,
     GrizzlyContextSetupLocustMessages,
     GrizzlyContextState,
@@ -23,19 +22,23 @@ from grizzly.context import (
     GrizzlyContextTasksTmp,
     load_configuration_file,
 )
-
+from grizzly.locust import FixedUsersDispatcher
+from grizzly.tasks import AsyncRequestGroupTask, ConditionalTask, ExplicitWaitTask, LogMessageTask, LoopTask, RequestTask
 from grizzly.types import MessageDirection, RequestMethod
-from grizzly.tasks import LogMessageTask, RequestTask, WaitTask, AsyncRequestGroupTask, LoopTask, ConditionalTask
+from grizzly.types.behave import Scenario
+from tests.helpers import ANY, TestTask, get_property_decorated_attributes, rm_rf
 
+if TYPE_CHECKING:  # pragma: no cover
+    from _pytest.tmpdir import TempPathFactory
 
-from tests.helpers import TestTask, get_property_decorated_attributes
-from tests.fixtures import BehaveFixture, GrizzlyFixture, RequestTaskFixture
+    from grizzly.types.locust import Environment, Message
+    from tests.fixtures import BehaveFixture, GrizzlyFixture, RequestTaskFixture
 
 
 def test_load_configuration_file(tmp_path_factory: TempPathFactory) -> None:
     configuration_file = tmp_path_factory.mktemp('configuration_file') / 'configuration.yaml'
     try:
-        configuration_file.write_text('''
+        configuration_file.write_text("""
             configuration:
                 sut:
                     host: 'https://backend.example.com'
@@ -47,16 +50,16 @@ def test_load_configuration_file(tmp_path_factory: TempPathFactory) -> None:
                             username: username
                             password: password
                             redirect_uri: "https://www.example.com/authenticated"
-        ''')
+        """)
 
         assert load_configuration_file() == {}
 
-        environ['GRIZZLY_CONFIGURATION_FILE'] = '/tmp/does-not-exist.yaml'
+        environ['GRIZZLY_CONFIGURATION_FILE'] = '/tmp/does-not-exist.yaml'  # noqa: S108
 
         with pytest.raises(SystemExit):
             load_configuration_file()
 
-        [name, _] = path.splitext(str(configuration_file))
+        name = configuration_file.name
 
         environ['GRIZZLY_CONFIGURATION_FILE'] = f'{name}.json'
 
@@ -74,33 +77,35 @@ def test_load_configuration_file(tmp_path_factory: TempPathFactory) -> None:
             'sut.auth.user.redirect_uri': 'https://www.example.com/authenticated',
         }
     finally:
-        shutil.rmtree(path.dirname(str(configuration_file)))
-        try:
+        rm_rf(configuration_file.parent)
+        with suppress(KeyError):
             del environ['GRIZZLY_CONFIGURATION_FILE']
-        except KeyError:
-            pass
 
 
 class TestGrizzlyContextSetup:
     def test(self, behave_fixture: BehaveFixture) -> None:
         behave = behave_fixture.context
-        grizzly = getattr(behave, 'grizzly')
+        grizzly = cast('GrizzlyContext', behave.grizzly)
         grizzly_setup = grizzly.setup
 
-        expected_properties: Dict[str, Optional[Tuple[Any, Any]]] = {
+        expected_properties: dict[str, tuple[Any, Any] | None] = {
             'log_level': ('INFO', 'DEBUG'),
-            'user_count': (0, 10),
+            'user_count': (None, 10),
             'spawn_rate': (None, 2),
             'timespan': (None, '10s'),
             'statistics_url': (None, 'influxdb://influx.example.org/grizzly'),
             'global_context': ({}, {'test': 'hello world'}),
             'locust': None,
+            'dispatcher_class': (None, FixedUsersDispatcher),
+            'hooks': ([], []),
+            'wait_for_spawning_complete': (None, 10.0),
         }
 
         expected_attributes = list(expected_properties.keys())
         expected_attributes.sort()
 
-        assert grizzly_setup is not None and isinstance(grizzly_setup, GrizzlyContextSetup)
+        assert grizzly_setup is not None
+        assert isinstance(grizzly_setup, GrizzlyContextSetup)
 
         for test_attribute_name, test_attribute_values in expected_properties.items():
             assert hasattr(grizzly_setup, test_attribute_name), f'attribute {test_attribute_name} does not exist in GrizzlyContextSetup'
@@ -128,10 +133,10 @@ class TestGrizzlyContextSetupLocustMessages:
         assert isinstance(context, dict)
         assert context == {}
 
-        def callback(environment: Environment, msg: Message, **kwargs: Dict[str, Any]) -> None:
+        def callback(environment: Environment, msg: Message, *args: Any, **kwargs: Any) -> None:
             pass
 
-        def callback_ack(environment: Environment, msg: Message, **kwargs: Dict[str, Any]) -> None:
+        def callback_ack(environment: Environment, msg: Message, *args: Any, **kwargs: Any) -> None:
             pass
 
         context.register(MessageDirection.SERVER_CLIENT, 'test_message', callback)
@@ -148,7 +153,7 @@ class TestGrizzlyContextSetupLocustMessages:
             },
             MessageDirection.CLIENT_SERVER: {
                 'test_message_ack': callback_ack,
-            }
+            },
         }
 
 
@@ -157,18 +162,13 @@ class TestGrizzlyContextState:
         state = GrizzlyContextState()
 
         expected_properties = {
-            'spawning_complete': (False, True),
-            'background_section_done': (False, True),
-            'variables': ({}, {'test': 'hello'}),
+            'spawning_complete': (ANY(Semaphore), None),
+            'background_done': (False, True),
             'configuration': ({}, {'sut.host': 'http://example.com'}),
-            'alias': ({}, {'AtomicIntegerIncrementer.test': 'auth.randomseed'}),
             'verbose': (False, True),
-            'persistent': ({}, {'AtomicIntegerIncrementer.persist': '1 | step=10, persist=True'}),
         }
-        actual_attributes = list(state.__dict__.keys())
-        actual_attributes.sort()
-        expected_attributes = list(expected_properties.keys())
-        expected_attributes.sort()
+        actual_attributes = sorted(state.__dict__.keys())
+        expected_attributes = sorted(expected_properties.keys())
 
         assert actual_attributes == expected_attributes
 
@@ -176,13 +176,18 @@ class TestGrizzlyContextState:
             assert test_attribute_name in actual_attributes
             assert hasattr(state, test_attribute_name)
             assert getattr(state, test_attribute_name) == default_value
-            setattr(state, test_attribute_name, test_value)
-            assert getattr(state, test_attribute_name) == test_value
 
-    def test_configuration(self, tmp_path_factory: TempPathFactory) -> None:
+            if test_value is not None:
+                setattr(state, test_attribute_name, test_value)
+                assert getattr(state, test_attribute_name) == test_value
+
+    def test_configuration(self, grizzly_fixture: GrizzlyFixture, tmp_path_factory: TempPathFactory) -> None:
+        grizzly = grizzly_fixture.grizzly
+        grizzly.scenarios.create(grizzly_fixture.behave.create_scenario('test-1'))
+
         configuration_file = tmp_path_factory.mktemp('configuration_file') / 'configuration.yaml'
         try:
-            configuration_file.write_text('''
+            configuration_file.write_text("""
                 configuration:
                     sut:
                         host: 'https://backend.example.com'
@@ -194,7 +199,7 @@ class TestGrizzlyContextState:
                                 username: username
                                 password: password
                                 redirect_uri: "https://www.example.com/authenticated"
-            ''')
+            """)
 
             state = GrizzlyContextState()
 
@@ -216,33 +221,33 @@ class TestGrizzlyContextState:
             }
 
         finally:
-            shutil.rmtree(path.dirname(str(configuration_file)))
-            try:
+            rm_rf(configuration_file.parent)
+            with suppress(KeyError):
                 del environ['GRIZZLY_CONFIGURATION_FILE']
-            except KeyError:
-                pass
 
 
 class TestGrizzlyContext:
     def test(self, behave_fixture: BehaveFixture) -> None:
         behave = behave_fixture.context
-        grizzly = getattr(behave, 'grizzly')
-        assert grizzly is not None
+        grizzly = cast('GrizzlyContext', behave.grizzly)
         assert isinstance(grizzly, GrizzlyContext)
 
-        second_grizzly = GrizzlyContext()
+        from grizzly.context import grizzly as second_grizzly
 
         assert grizzly is second_grizzly
 
+        grizzly.scenarios.create(behave_fixture.create_scenario('test scenario'))
+
         expected_attributes = [
             'setup',
             'state',
             'scenario',
             'scenarios',
+            'events',
         ]
         expected_attributes.sort()
 
-        actual_attributes = list(get_property_decorated_attributes(grizzly.__class__))
+        actual_attributes = list(get_property_decorated_attributes(grizzly.__class__)) + list(filter(lambda d: not d.startswith('_'), grizzly.__dict__))
         actual_attributes.sort()
 
         for test_attribute in expected_attributes:
@@ -252,24 +257,12 @@ class TestGrizzlyContext:
         assert callable(getattr(grizzly, 'scenarios', None))
         assert expected_attributes == actual_attributes
 
-    def test_destroy(self, behave_fixture: BehaveFixture) -> None:
-        behave = behave_fixture.context
-        grizzly = getattr(behave, 'grizzly')
-        assert grizzly is GrizzlyContext()
-
-        GrizzlyContext.destroy()
-
-        with pytest.raises(ValueError):
-            GrizzlyContext.destroy()
-
-        grizzly = GrizzlyContext()
-
-        assert grizzly is GrizzlyContext()
-
 
 class TestGrizzlyContextScenarios:
-    def test(self) -> None:
-        scenarios = GrizzlyContextScenarios()
+    def test(self, grizzly_fixture: GrizzlyFixture) -> None:
+        # setup env variable GRIZZLY_CONTEXT_ROOT
+        _ = grizzly_fixture()
+        scenarios = GrizzlyContextScenarios(grizzly_fixture.grizzly)
 
         assert issubclass(scenarios.__class__, list)
         assert len(scenarios) == 0
@@ -279,7 +272,7 @@ class TestGrizzlyContextScenarios:
         assert len(scenarios) == 1
         assert scenarios[-1].name == 'test-1'
         assert scenarios[-1].description == 'test-1'
-        assert scenarios[-1].class_name == 'test-1_001'
+        assert scenarios[-1].class_name == 'IteratorScenario_001'
         assert scenarios[-1].behave is behave_scenario
 
         behave_scenario = Scenario(filename=None, line=None, keyword='', name='test-2')
@@ -287,50 +280,51 @@ class TestGrizzlyContextScenarios:
         assert len(scenarios) == 2
         assert scenarios[-1].name == 'test-2'
         assert scenarios[-1].description == 'test-2'
-        assert scenarios[-1].class_name == 'test-2_002'
+        assert scenarios[-1].class_name == 'IteratorScenario_002'
         assert scenarios[-1].behave is behave_scenario
 
         assert len(scenarios()) == 2
 
-        assert scenarios.find_by_class_name('test-2_002') is scenarios[-1]
+        assert scenarios.find_by_class_name('IteratorScenario_002') is scenarios[-1]
         assert scenarios.find_by_name('test-1') is scenarios[-2]
+        assert scenarios.find_by_class_name('IteratorScenario_001') is scenarios[-2]
 
 
 class TestGrizzlyContextScenario:
-    @pytest.mark.parametrize('index', [
-        1,
-        12,
-        99,
-        104,
-        999,
-        1004,
-    ])
-    def test(self, index: int) -> None:
-        scenario = GrizzlyContextScenario(index)
+    @pytest.mark.parametrize(
+        'index',
+        [
+            1,
+            12,
+            99,
+            104,
+            999,
+            1004,
+        ],
+    )
+    def test(self, index: int, behave_fixture: BehaveFixture) -> None:
+        scenario = GrizzlyContextScenario(index, behave=behave_fixture.create_scenario('Test'), grizzly=behave_fixture.grizzly)
         identifier = f'{index:03}'
 
         assert scenario.index == index
         assert scenario.identifier == identifier
-        assert not hasattr(scenario, 'name')
+        assert scenario.name == 'Test'
+        assert scenario.description == 'Test'
         assert not hasattr(scenario, 'user_class_name')
         assert scenario.iterations == 1
         assert scenario.context == {}
         assert scenario.tasks() == []
         assert getattr(scenario, 'pace', '') is None
         assert isinstance(scenario.validation, GrizzlyContextScenarioValidation)
-        assert not scenario.failure_exception
+        assert scenario.failure_handling.get(None, None) is None
+        assert scenario.class_type == 'IteratorScenario'
 
-        scenario.name = 'Test'
-        assert scenario.class_name == f'Test_{identifier}'
-
-        scenario.name = f'Test_{identifier}'
-        assert scenario.class_name == f'Test_{identifier}'
+        assert scenario.class_name == f'IteratorScenario_{identifier}'
 
         assert not scenario.should_validate()
 
-    def test_tasks(self, request_task: RequestTaskFixture) -> None:
-        scenario = GrizzlyContextScenario(1)
-        scenario.name = 'TestScenario'
+    def test_tasks(self, request_task: RequestTaskFixture, behave_fixture: BehaveFixture) -> None:
+        scenario = GrizzlyContextScenario(1, behave=behave_fixture.create_scenario('TestScenario'), grizzly=behave_fixture.grizzly)
         scenario.context['host'] = 'test'
         scenario.user.class_name = 'TestUser'
         request = request_task.request
@@ -338,17 +332,14 @@ class TestGrizzlyContextScenario:
         scenario.tasks.add(request)
 
         assert scenario.tasks() == [request]
-        assert isinstance(scenario.tasks()[-1], RequestTask) and scenario.tasks()[-1].scenario is scenario
 
         second_request = RequestTask(RequestMethod.POST, name='Second Request', endpoint='/api/test/2')
         second_request.source = '{"hello": "world!"}'
-        assert isinstance(second_request.template, Template)
 
         scenario.tasks.add(second_request)
         assert scenario.tasks() == [request, second_request]
-        assert isinstance(scenario.tasks()[-1], RequestTask) and scenario.tasks()[-1].scenario is scenario
 
-        wait_task = WaitTask(time_expression='1.337')
+        wait_task = ExplicitWaitTask(time_expression='1.337')
         scenario.tasks.add(wait_task)
         assert scenario.tasks() == [request, second_request, wait_task]
 
@@ -358,15 +349,13 @@ class TestGrizzlyContextScenario:
 
     def test_scenarios(self, behave_fixture: BehaveFixture) -> None:
         behave = behave_fixture.context
-        grizzly = cast(GrizzlyContext, behave.grizzly)
+        grizzly = cast('GrizzlyContext', behave.grizzly)
         assert len(grizzly.scenarios()) == 0
-        assert grizzly.state.variables == {}
 
         grizzly.scenarios.create(behave_fixture.create_scenario('test1'))
         grizzly.scenario.user.class_name = 'TestUser'
         first_scenario = grizzly.scenario
         assert len(grizzly.scenarios()) == 1
-        assert grizzly.state.variables == {}
         assert grizzly.scenario.name == 'test1'
 
         grizzly.scenario.context['host'] = 'http://test:8000'
@@ -408,9 +397,8 @@ class TestGrizzlyContextTasksTmp:
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is async_group
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='async_group is already in stack'):
             tmp.async_group = AsyncRequestGroupTask(name='async-group-2')
-        assert str(ae.value) == 'async_group is already in stack'
 
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is async_group
@@ -419,9 +407,8 @@ class TestGrizzlyContextTasksTmp:
 
         assert len(tmp.__stack__) == 0
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='async_group is not in stack'):
             tmp.async_group = None
-        assert str(ae.value) == 'async_group is not in stack'
 
         assert len(tmp.__stack__) == 0
 
@@ -436,9 +423,8 @@ class TestGrizzlyContextTasksTmp:
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is conditional
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='conditional is already in stack'):
             tmp.conditional = ConditionalTask('cond-2', '{{ value | int > 10 }}')
-        assert str(ae.value) == 'conditional is already in stack'
 
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is conditional
@@ -447,9 +433,8 @@ class TestGrizzlyContextTasksTmp:
 
         assert len(tmp.__stack__) == 0
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='conditional is not in stack'):
             tmp.conditional = None
-        assert str(ae.value) == 'conditional is not in stack'
 
         assert len(tmp.__stack__) == 0
 
@@ -459,17 +444,16 @@ class TestGrizzlyContextTasksTmp:
 
         assert len(tmp.__stack__) == 0
 
-        grizzly.state.variables['loop_value'] = 'none'
+        grizzly.scenario.variables['loop_value'] = 'none'
 
-        loop = LoopTask(grizzly, 'loop-1', '["hello", "world"]', "loop_value")
+        loop = LoopTask('loop-1', '["hello", "world"]', 'loop_value')
         tmp.loop = loop
 
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is loop
 
-        with pytest.raises(AssertionError) as ae:
-            tmp.loop = LoopTask(grizzly, 'loop-2', '["hello", "world"]', "loop_value")
-        assert str(ae.value) == 'loop is already in stack'
+        with pytest.raises(AssertionError, match='loop is already in stack'):
+            tmp.loop = LoopTask('loop-2', '["hello", "world"]', 'loop_value')
 
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is loop
@@ -478,15 +462,14 @@ class TestGrizzlyContextTasksTmp:
 
         assert len(tmp.__stack__) == 0
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='loop is not in stack'):
             tmp.loop = None
-        assert str(ae.value) == 'loop is not in stack'
 
         assert len(tmp.__stack__) == 0
 
     def test___stack__(self, grizzly_fixture: GrizzlyFixture) -> None:
         grizzly = grizzly_fixture.grizzly
-        grizzly.state.variables['loop_value'] = 'none'
+        grizzly.scenario.variables['loop_value'] = 'none'
 
         tmp = GrizzlyContextTasksTmp()
 
@@ -498,7 +481,7 @@ class TestGrizzlyContextTasksTmp:
         assert len(tmp.__stack__) == 1
         assert tmp.__stack__[-1] is conditional
 
-        loop = LoopTask(grizzly, 'loop-1', '["hello", "world"]', "loop_value")
+        loop = LoopTask('loop-1', '["hello", "world"]', 'loop_value')
         tmp.loop = loop
 
         assert len(tmp.__stack__) == 2
@@ -520,9 +503,8 @@ class TestGrizzlyContextTasksTmp:
         assert len(tmp.__stack__) == 3
         assert tmp.__stack__[-1] is async_group
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='loop is not last in stack'):
             tmp.loop = None
-        assert str(ae.value) == 'loop is not last in stack'
 
         assert len(tmp.__stack__) == 3
         assert tmp.__stack__[-1] is async_group
@@ -532,9 +514,8 @@ class TestGrizzlyContextTasksTmp:
         assert len(tmp.__stack__) == 2
         assert tmp.__stack__[-1] is loop
 
-        with pytest.raises(AssertionError) as ae:
+        with pytest.raises(AssertionError, match='conditional is not last in stack'):
             tmp.conditional = None
-        assert str(ae.value) == 'conditional is not last in stack'
 
         tmp.loop = None
 
@@ -547,23 +528,16 @@ class TestGrizzlyContextTasksTmp:
 
 class TestGrizzlyContextTasks:
     def test___init__(self) -> None:
-        scenario = GrizzlyContextScenario(2)
-        scenario.name = scenario.description = 'test scenario'
+        tasks = GrizzlyContextTasks()
 
-        tasks = GrizzlyContextTasks(scenario)
-
-        assert tasks.scenario is scenario
-        assert isinstance(tasks._tmp, GrizzlyContextTasksTmp)
+        assert isinstance(tasks.tmp, GrizzlyContextTasksTmp)
         assert tasks.tmp is tasks._tmp
 
     def test___call__(self, grizzly_fixture: GrizzlyFixture) -> None:
         grizzly = grizzly_fixture.grizzly
-        grizzly.state.variables['loop_value'] = 'none'
+        grizzly.scenario.variables['loop_value'] = 'none'
 
-        scenario = GrizzlyContextScenario(2)
-        scenario.name = scenario.description = 'test scenario'
-
-        tasks = GrizzlyContextTasks(scenario)
+        tasks = GrizzlyContextTasks()
 
         assert len(tasks()) == 0
 
@@ -574,7 +548,7 @@ class TestGrizzlyContextTasks:
         assert len(tasks.tmp.__stack__) == 0
 
         tasks.tmp.conditional = ConditionalTask('cond-1', '{{ value | int > 0 }}')
-        tasks.tmp.conditional.switch(True)
+        tasks.tmp.conditional.switch(pointer=True)
 
         assert len(tasks()) == 0
         assert len(tasks.tmp.__stack__) == 1
@@ -584,7 +558,7 @@ class TestGrizzlyContextTasks:
 
         assert len(tasks()) == 1
 
-        tasks.tmp.loop = LoopTask(grizzly, 'loop-1', '["hello", "world"]', "loop_value")
+        tasks.tmp.loop = LoopTask('loop-1', '["hello", "world"]', 'loop_value')
 
         assert len(tasks()) == 0
         assert len(tasks.tmp.__stack__) == 2
